!###############################################################
!#
!# Note: Scalar Mediator Physics 
!#
!# Author: Zuowei Liu, Yushan Su, Yue-Lin Sming Tsai and Binrong Yu
!# Email: zuoweiliu@nju.edu.cn, 1034026870@qq.com, smingtsai@gmail.com, 944174541@qq.com                                                                                
!# Date: 2017-02-18
!###############################################################


module DirectDetLike
  use MathLib
  implicit none

    real*8 LUX14to16__R95(50),LUX14to16__m95(50) 
    real*8 LUX13__R95(50),LUX13__m95(50)
    real*8 PandaX16__m95(50),PandaX16__R95(50)
    real*8 PandaX16__fmx(50) 
    real*8 PandaX16__R(100),PandaX16__chisq(100)
    real*8 PandaXRun9__R(100),PandaXRun9__chisq(100)
    real*8 PandaXRun10__R(100),PandaXRun10__chisq(100)    
    real*8 PandaXRun9Run10__m95(40),PandaXRun9Run10__hmx(40)

    real*8 XENON1T__m95(40),XENON1T__R95(40)
    real*8 XENON1T__gmx(40)
    real*8 XENON1T__R(100),XENON1T__chisq(100)


    logical :: DirectDetLike__loaded

contains

  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  !  Load Likelihood Table
  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine Read_DDLikeTable()
    implicit none 
    character (LEN=20) :: buf
    integer i
    real*8 Rstat,xCL(2)
      
    
    !### reading the data ###
    open(unit=49,file='maps/LUX2013_mx_vs_R95.txt',status='old')
    read(49,*)buf
    do i=1,50,1
       read(unit=49,fmt=*) LUX13__m95(i),LUX13__R95(i)
    enddo 
    close(49)

    open(unit=49,file='maps/LUX201416_mx_vs_R95.txt',status='old')
    read(49,*)buf
    do i=1,50,1
       read(unit=49,fmt=*) LUX14to16__m95(i),LUX14to16__R95(i)
    enddo 
    close(49)

    open(unit=49,file='maps/PandaX16_mx_vs_R95.txt',status='old')
    read(49,*)buf
    do i=1,50,1
       read(unit=49,fmt=*) PandaX16__m95(i),PandaX16__R95(i)
    enddo 
    close(49)

    open(unit=49,file='maps/PandaX16_R_vs_chisq.txt',status='old')
    read(49,*)buf
    do i=1,100,1
       read(unit=49,fmt=*) PandaX16__R(i),PandaX16__chisq(i)
    enddo 
    close(49)
    
    open(unit=49,file='maps/PandaXRun9_R_vs_chisq.txt',status='old')
    read(49,*)buf
    do i=1,100,1
       read(unit=49,fmt=*) PandaXRun9__R(i),PandaXRun9__chisq(i)
    enddo 
    close(49)  

    open(unit=49,file='maps/PandaXRun10_R_vs_chisq.txt',status='old')
    read(49,*)buf
    do i=1,100,1
       read(unit=49,fmt=*) PandaXRun10__R(i),PandaXRun10__chisq(i)
    enddo 
    close(49)

    open(unit=49,file='maps/XENON1T_mx_vs_R95.txt',status='old')
    read(49,*)buf
    do i=1,40,1
       read(unit=49,fmt=*) XENON1T__m95(i),XENON1T__R95(i)
    enddo 
    close(49)

    open(unit=49,file='maps/XENON1T_R_vs_chisq.txt',status='old')
    read(49,*)buf
    do i=1,100,1
       read(unit=49,fmt=*) XENON1T__R(i),XENON1T__chisq(i)
    enddo 
    close(49)


    ! --- Find out fmx = Rstat@95/Rth@95 ---
    do i=1,50,1 
      call FindCL_range(100,PandaX16__R,PandaX16__chisq,&
                        PandaX16__chisq(1)+2.71d0,xCL)
      Rstat=xCL(2)
      PandaX16__fmx(i)=Rstat/PandaX16__R95(i)
    enddo
    
    ! --- Find out gmx = Rstat@95/Rth@95 ---
    do i=1,40,1 
      call FindCL_range(100,XENON1T__R,XENON1T__chisq,&
                        XENON1T__chisq(1)+2.71d0,xCL)
      Rstat=xCL(2)
      XENON1T__gmx(i)=Rstat/XENON1T__R95(i)
    enddo    
    
    ! --- Load hmx = sigmath@95/sigmastat@95 ---
    open(unit=49,file='maps/PandaXRun9Run10_hmx.txt',status='old')
    read(49,*)buf
    do i=1,40,1
       read(unit=49,fmt=*) PandaXRun9Run10__m95(i),PandaXRun9Run10__hmx(i)
    enddo 
    close(49)
    
    ! Flag to tell the users the table is loaded
    write(*,*) '... Loaded DirectDetLike Tables <<< '

    DirectDetLike__loaded=.true.
    return 
  end subroutine




  subroutine DDexp_chisq(What_experiment,mx,R,chisq)
    implicit none
    CHARACTER(LEN=*), intent(in) :: What_experiment
    real*8, intent(in) :: mx,R
    real*8, intent(out) :: chisq
    real*8 R95,sigma,fmx,gmx,hmx,Rstat


    ! --- check whether the table loaded or not
    if (.not. DirectDetLike__loaded) call Read_DDLikeTable()


    !--- Find out the chisq based on the prepared tables ---
    if (trim(What_experiment).eq. 'PandaX-II') then 
      ! find fmx = Rstat@95/Rth@95 at m
      call linInt2p(PandaX16__m95,PandaX16__fmx,50,.true., &
                    mx,fmx,.false.,.true.)
      ! From Rstat list to obtain chisq
      Rstat= fmx*R     
      call linInt2p(PandaX16__R,PandaX16__chisq,100,.false., &
                    Rstat,chisq,.false.,.true.)

    elseif (trim(What_experiment).eq. 'PandaXRun9') then 
      ! find hmx = sigmath@95/sigmastat@95 at m
      call linInt2p(PandaXRun9Run10__m95,PandaXRun9Run10__hmx,40,.true., &
                    mx,hmx,.false.,.true.)
      ! From Rstat list to obtain chisq
      Rstat= R/hmx  
      call linInt2p(PandaXRun9__R,PandaXRun9__chisq,100,.false., &
                    Rstat,chisq,.false.,.true.)

    elseif (trim(What_experiment).eq. 'PandaXRun10') then 
      ! find hmx = sigmath@95/sigmastat@95 at m
      call linInt2p(PandaXRun9Run10__m95,PandaXRun9Run10__hmx,40,.true., &
                    mx,hmx,.false.,.true.)
      ! From Rstat list to obtain chisq
      Rstat= R/hmx  
      call linInt2p(PandaXRun10__R,PandaXRun10__chisq,100,.false., &
                    Rstat,chisq,.false.,.true.)
      
    elseif (trim(What_experiment).eq. 'XENON1T') then 
      ! find gmx = Rstat@95/Rth@95 at m
      call linInt2p(XENON1T__m95,XENON1T__gmx,40,.true., &
                    mx,gmx,.false.,.true.)
      ! From Rstat list to obtain chisq
      Rstat= gmx*R     
      call linInt2p(XENON1T__R,XENON1T__chisq,100,.false., &
                    Rstat,chisq,.false.,.true.)
      
    elseif (trim(What_experiment).eq. 'LUX-2013') then 
      ! LUX13 R95@mx
      call linInt2p(LUX13__m95,LUX13__R95,50,.true., &
                    mx,R95,.false.,.true.)
      sigma=R95/dsqrt(2.71d0) 
      chisq=(R/sigma)**2

    elseif (trim(What_experiment).eq. 'LUX-2014-2016') then 
      ! LUX13 R95@mx
      call linInt2p(LUX14to16__m95,LUX14to16__R95,50,.true., &
                    mx,R95,.false.,.true.)
      sigma=R95/dsqrt(2.71d0) 
      chisq=(R/sigma)**2
    else
       write(*,*)'No likelihood exist for '//trim(What_experiment)
       write(*,*)'Please add it to module DirectDet_likelihood.f90.'
       STOP
    endif


    return 
  end subroutine



  subroutine FindCL_range(n,x,chisqs,chisqCL,xCL)
    implicit none
    integer, intent(in) :: n
    real*8, intent(in) :: x(n),chisqs(n)
    real*8, intent(in) :: chisqCL
    real*8, intent(out) :: xCL(2) 
    integer i
    real*8 y(2)

    y(:)=0d0

    do i=1,n-1,1


      if (chisqCL>=chisqs(i) .and. chisqCL<=chisqs(i+1)) then 
        call Between_two_pts(chisqs(i),x(i),&
                             chisqs(i+1),x(i+1),.true., &
                             chisqCL,y(1))    
      endif

      if (chisqCL<=chisqs(i) .and. chisqCL>=chisqs(i+1)) then 
        call Between_two_pts(chisqs(i),x(i),&
                             chisqs(i+1),x(i+1),.true., &
                             chisqCL,y(2))    
      endif     

    enddo

    if (y(1)>y(2)) then 
      xCL(1)=y(2)
      xCL(2)=y(1)
    else
      xCL(1)=y(1)
      xCL(2)=y(2)
    endif

    return 
  end subroutine


!###############################################################
end module DirectDetLike

