module prior_type
  use randdp
  implicit none


  contains


!##################################################################        
  subroutine defaultPrior(WhatPrior,vmin,vmax,frac,ans)
!##################################################################        
    implicit none
    CHARACTER(LEN=*) WhatPrior
    logical use_abs
    real*8,parameter :: es=1d-8
    real*8 vmin,vmax,frac,ans
    real*8 logvmin,logvmax

    if (trim(WhatPrior) .eq. 'log' .and. vmin<0d0 ) then 
      write(*,*)'Note log prior in defaultPrior subroutine '
      write(*,*)'does not support negative value. Please use Prior_in_absV instead.'
      write(*,*)trim(WhatPrior),vmin,vmax
      STOP 
    endif


    if (trim(WhatPrior) .eq. 'log') then 

      if (vmin==0d0) then 
        logvmin=dlog10(es)
      else
        logvmin=dlog10(vmin)
      endif
      logvmax=dlog10(vmax)    

      ans=(logvmax-logvmin)*frac + logvmin
      ans=10d0**ans

    elseif (trim(WhatPrior) .eq. 'flat') then 
      ans=(vmax-vmin)*frac + vmin

    else 

      write(*,*)'your choice not support in default!!!'
      STOP

    endif

    return
  end subroutine defaultPrior


!##################################################################        
  subroutine SmingMixPrior(vmin,vmax,frac,ans)
!##################################################################        
    implicit none
    real*8 takeOne
    real*8 vmin,vmax,frac,ans

    takeOne=dprand() 

    if (takeOne>0.5d0) then 
      call defaultPrior('log',vmin,vmax,frac,ans)
    else
      call defaultPrior('flat',vmin,vmax,frac,ans)
    endif 

    return
  end subroutine SmingMixPrior

!##################################################################        
  subroutine SmingGau_prior(f0,f1,cv,width,ans)
!##################################################################        
    implicit none
    real*8 cv,width,ans
    real*8 f0,f1,chi
    chi=dsqrt(dlog(f0)*(-2d0))*dcos( f1* dasin(1.0d0)*4d0)
    ans=cv+chi*width

    return
  end subroutine

!##################################################################        
  subroutine Prior_in_absV(WhatPrior,absvmin,absvmax,frac,ans)
!##################################################################        
    implicit none
    CHARACTER(LEN=*) WhatPrior
    real*8 Vsign
    real*8 absvmin,absvmax,frac,ans

    if (trim(WhatPrior) .eq. 'mix') then   
       call SmingMixPrior(absvmin,absvmax,frac,ans)
    else
       call defaultPrior(WhatPrior,absvmin,absvmax,frac,ans)
    endif


    Vsign=dprand() 
    if (Vsign>0.5d0) then 
      ans=-1.0d0*ans  
    endif

    return
  end subroutine Prior_in_absV

!##################################################################        
  end module prior_type

